/* life3d.h
 * Nadim Abdo 1998
 *
 * life3d
 *
 * For Operating Systems 304-427B Assignment #3
 */


#ifndef _life3d_h_
#define _life3d_h_

/*Number of cells on each side*/
#define N 6
/*Number of faces*/
#define FACES 6
/*Each face has 4 edges*/
#define NUM_EDGES 4

/*Face ID's*/
#define FACE1 1-1
#define FACE2 2-1
#define FACE3 3-1
#define FACE4 4-1
#define FACE5 5-1
#define FACE6 6-1

/*Edge identifiers*/
#define NORTH 0
#define EAST  1
#define SOUTH 2
#define WEST  3

/*Cell face info*/
typedef struct tag_ball_state
{
  int cells[FACES][N][N];
} life_state;


typedef enum {RevDir,NormDir} AxisDir;

/*Defines an edge in terms of it's face ID and
  location, N,S,E,W*/
typedef struct
{
  int face;
  int edge;
  /*If the coordinate axes of the two neigbouring faces
    are in opposing directions, this is marked by setting
    the direction to RevDir*/
  AxisDir dir;
} neighbour_edge;



/*neigbouring edges information*/
typedef struct tag_face_neigbourinfo
{
  /*These are ordered in the following way:
    top neighbour, right neighbour, bottom neighbour, left
    (relative to the origins given on the course www page
  */

  neighbour_edge nbors[4];
} face_neighbourinfo;



/*Neigbourhood data (1 entry per face)*/
static face_neighbourinfo face_neighbours[] =
{
  { /*Face 1's neighbourhood*/
    {{FACE5,NORTH,NormDir},{FACE2,WEST,NormDir},{FACE6,NORTH,NormDir},{FACE4,EAST,NormDir}}
  },

  { /*Face 2's neighbourhood*/
    {{FACE5,EAST,NormDir},{FACE3,WEST,NormDir},{FACE6,EAST,NormDir},{FACE1,EAST,NormDir}}
  },

  { /*Face 3's neighbourhood*/
    {{FACE5,SOUTH,RevDir},{FACE4,WEST,NormDir},{FACE6,SOUTH,RevDir},{FACE2,EAST,NormDir}}
  },

  { /*Face 4's neighbourhood*/
    {{FACE5,WEST,RevDir},{FACE1,WEST,NormDir},{FACE6,WEST,RevDir},{FACE3,EAST,NormDir}}
  },

  { /*Face 5's neighbourhood*/
    {{FACE1,NORTH,NormDir},{FACE2,NORTH,NormDir},{FACE3,NORTH,RevDir},{FACE4,NORTH,RevDir}}
  },
  
  { /*Face 6's neighbourhood*/
    {{FACE1,SOUTH,NormDir},{FACE2,SOUTH,NormDir},{FACE3,SOUTH,RevDir},{FACE4,SOUTH,RevDir}}
  }
};


/* =================================================== */
/* =============== Global variables ================== */

/*Pointer to shared memory buffer*/
life_state* pLifeState;


/* shared memory identifier for each face */
int shmid;

/* semaphore identifiers */

/* semaphore that drops to 0 when there are no more processes
   left in the game, triggers call to cleanup IPC resources */
int sem_cleanup_watchdog;
/* semaphores that protect access to the shared memory area of
   each face  */
int sem_shared_mem;
/* semaphore that synchronizes the start of a time slice */
int sem_time_slice_start;
/* semaphore that synchronizes the end of a time slice */
int sem_time_slice_end;
/* semaphore that counts number of faces that have
   reached a certain point in their update code*/
int sem_num_updated_faces;
/* semaphore that ensures all faces are done commiting next state
   to the current state*/
int sem_faces_commited_next_state;

#endif
